<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xsd="http://www.w3.org/2001/XMLSchema">
	<xsl:output method="xml" encoding="UTF-8" indent="yes"/>
	<!-- ==================================================== -->
	<xsl:template name="getTargetNameSpacePrefix">
		<xsl:param name="list"/>
		<xsl:if test="$list != ''">
			<xsl:variable name="first" select="substring-before($list, '/')"/>
			<xsl:variable name="rest" select="substring-after($list, '/')"/>
			<xsl:choose>
				<xsl:when test="contains($rest,'/')">
					<xsl:call-template name="getTargetNameSpacePrefix">
						<xsl:with-param name="list" select="$rest"/>
					</xsl:call-template>
				</xsl:when>
				<xsl:when test="$rest =''">
					<xsl:value-of select="$first"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$rest"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:if>
	</xsl:template>
	<!-- ==================================================== -->
	<xsl:template name="getPathPrefix">
		<xsl:param name="file"/>
		<xsl:if test="contains($file,'/')">
			<xsl:variable name="pre" select="substring-before($file,'/')"/>
			<xsl:variable name="suf" select="substring-after($file,'/')"/>
			<xsl:choose>
				<xsl:when test="contains($suf,'/')">
					<xsl:value-of select="$pre"/><xsl:text>/</xsl:text>
					<xsl:call-template name="getPathPrefix">
						<xsl:with-param name="file" select="$suf"/>
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:variable name="path">
						<xsl:call-template name="removeSuffix">
							<xsl:with-param name="name" select="$file"/>
							<xsl:with-param name="suf" select="$suf"/>
						</xsl:call-template>
					</xsl:variable>
					<xsl:value-of select="$path"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:if>
	</xsl:template>
	<!-- ==================================================== -->
	<xsl:template name="removePrefix">
		<xsl:param name="name"/>
		<xsl:param name="pre"/>
		<xsl:variable name="npName">
			<xsl:choose>
				<xsl:when test="contains($name,$pre)">
					<xsl:value-of select="substring-after($name,$pre)"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$name"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:value-of select="$npName"/>
	</xsl:template>
	<!-- ==================================================== -->
	<xsl:template name="removeSuffix">
		<xsl:param name="name"/>
		<xsl:param name="suf"/>
		<xsl:variable name="nsName">
			<xsl:choose>
				<xsl:when test="contains($name,$suf)">
					<xsl:value-of select="substring-before($name,$suf)"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$name"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:value-of select="$nsName"/>
	</xsl:template>
	<!-- ==================================================== -->
	<xsl:template name="lowerLeading">
		<xsl:param name="name"/>
		<xsl:variable name="ch1" select="substring($name, 1, 1)"/>
		<xsl:variable name="lc1" select="translate($ch1,'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')"/>
		<xsl:value-of select="concat($lc1, substring($name, 2))"/>
	</xsl:template>
	<!-- ==================================================== -->
	<xsl:template name="uniqueList">
		<xsl:param name="list"/>
		<xsl:param name="sep"/>
		<xsl:param name="seen"/>
		<xsl:param name="pre">../base/</xsl:param>
		<xsl:if test="$list != ''">
			<xsl:variable name="first" select="substring-before($list, $sep)"/>
			<xsl:variable name="firstSep" select="concat($first,$sep)"/>
			<xsl:variable name="rest" select="substring-after($list, $sep)"/>
			<xsl:choose>
				<xsl:when test="contains($seen,$firstSep)">
					<xsl:call-template name="uniqueList">
						<xsl:with-param name="list" select="$rest"/>
						<xsl:with-param name="sep" select="$sep"/>
						<xsl:with-param name="seen" select="$seen"/>
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$firstSep"/>
					<xsl:variable name="nowSeen" select="concat($seen, $firstSep)"/>
					<xsl:call-template name="uniqueList">
						<xsl:with-param name="list" select="$rest"/>
						<xsl:with-param name="sep" select="$sep"/>
						<xsl:with-param name="seen" select="$nowSeen"/>
					</xsl:call-template>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:if>
	</xsl:template>
	<!-- ==================================================== -->
	<xsl:template name="getIncludedDocs">
		<xsl:param name="docName"/>
		<xsl:param name="usePre"/>
		<xsl:param name="seenList"/>
		<xsl:param name="sep">,</xsl:param>
		<xsl:value-of select="$docName"/>
		<xsl:text>,</xsl:text>
		<xsl:variable name="pathPre">
			<xsl:call-template name="getPathPrefix">
				<xsl:with-param name="file" select="$docName"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="callPathPre">
			<xsl:choose>
				<xsl:when test="$pathPre = '' or $pathPre = './' ">
					<xsl:value-of select="$usePre"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$pathPre"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:for-each select="document($docName, / )">
			<xsl:for-each select="//xsd:include | //xsd:import">
				<xsl:variable name="iDoc" select="@schemaLocation"/>
				<xsl:variable name="iPathPre">
					<xsl:call-template name="getPathPrefix">
						<xsl:with-param name="file" select="$iDoc"/>
					</xsl:call-template>
				</xsl:variable>
				<xsl:variable name="iDocSuf">
					<xsl:call-template name="removePrefix">
						<xsl:with-param name="name" select="$iDoc"/>
						<xsl:with-param name="pre" select="$iPathPre"/>
					</xsl:call-template>
				</xsl:variable>
				<xsl:variable name="usePathPre">
					<xsl:choose>
						<xsl:when test="$iPathPre = '' or $iPathPre = './' ">
							<xsl:value-of select="$callPathPre"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$iPathPre"/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>
				<xsl:variable name="uDoc">
					<xsl:value-of select="concat($usePathPre,$iDocSuf)"/>
				</xsl:variable>
				<xsl:variable name="uDocSep">
					<xsl:value-of select="concat($uDoc,$sep)"/>
				</xsl:variable>
				<xsl:if test="not(contains($seenList,$uDocSep))">
					<xsl:variable name="seenListPlus" select="concat($seenList,$uDocSep)"/>	
			 		<xsl:call-template name="getIncludedDocs">
						<xsl:with-param name="docName" select="$uDoc"/>
						<xsl:with-param name="usePre" select="$usePathPre"/>
						<xsl:with-param name="seenList" select="$seenListPlus"/>
					</xsl:call-template>
				</xsl:if>
			</xsl:for-each>
		</xsl:for-each>
	</xsl:template>
	<!-- ==================================================== -->
	<xsl:template name="getDocumentList">
		<xsl:param name="list"/>
		<xsl:param name="seenList"/>
		<xsl:param name="usePre"/>
		<xsl:if test="$list != ''">
			<xsl:variable name="first" select="substring-before($list, ',')"/>
			<xsl:variable name="rest" select="substring-after($list, ',')"/>
			<xsl:variable name="included">
				<xsl:call-template name="getIncludedDocs">
					<xsl:with-param name="docName" select="$first"/>
					<xsl:with-param name="usePre" select="$usePre"/>
					<xsl:with-param name="seenList" select="$seenList"/>
				</xsl:call-template>
			</xsl:variable>
			<xsl:value-of select="$included"/>
			<xsl:variable name="seenListIncluded" select="concat($seenList,$included)"/>
			<xsl:if test="contains($rest,',')">
				<xsl:call-template name="getDocumentList">
					<xsl:with-param name="list" select="$rest"/>
					<xsl:with-param name="seenList" select="$seenListIncluded"/>
					<xsl:with-param name="usePre" select="$usePre"/>
				</xsl:call-template>
			</xsl:if>
		</xsl:if>
	</xsl:template>
	<!-- ==================================================== -->
	<xsl:template name="getUniqueSchemaList">
		<xsl:param name="list"/>
		<xsl:param name="usePre"/>
		<xsl:variable name="allSchemas">
			<xsl:call-template name="getDocumentList">
				<xsl:with-param name="list" select="$list"/>
				<xsl:with-param name="usePre" select="$usePre"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="uniqueSchemas">
			<xsl:call-template name="uniqueList">
				<xsl:with-param name="list" select="$allSchemas"/>
				<xsl:with-param name="sep">,</xsl:with-param>
			</xsl:call-template>
		</xsl:variable>
		<xsl:value-of select="$uniqueSchemas"/> 
	</xsl:template>
	<!-- =========================================================== -->
	  <xsl:template name="Separator">
    <xsl:param name="comment" select="'=========='"/>
    <xsl:text>&#xA;</xsl:text>
    <xsl:text disable-output-escaping="yes">
       &lt;!-- ==================</xsl:text><xsl:value-of select="$comment"/><xsl:text disable-output-escaping="yes">===================== --&gt;
   </xsl:text>  
	<!-- =========================================================== -->
  </xsl:template>

	
</xsl:stylesheet>
